# Обращения (entry)

## Метод “entry”

Метод позволяет запрашивать информацию по Обращениям

<pre><code class="html">URL для вызова - http(s)://crm_url/developer/v2/entry?параметр=значение</code></pre>

<a id="list"></a>
### Запрос “list”

Запрос позволяет получить список сделок, доступных текущему сотруднику, в т.ч. с применением фильтров.

**Параметры запроса (не обязательные):**

- **offset** – страница вывода, с учетом того, что установлен лимит в 200 записей на страницу (по умолчанию offset = 0)
- **order** – поле, по которому будет производится сортировка списка (по умолчанию order = datum)
- **first** – направление сортировки (new – сначала новые, old – сначала старые). (по умолчанию first = new)

**Фильтры (не обязательные):**

- **user** – ограничение по пользователю (указывается логин пользователя)
- **dateStart** – начальная дата создания записи (формат - YYYY-MM-DD)
- **dateEnd** – конечная дата создания записи (не обязательно, формат - YYYY-MM-DD)
  - только dateStart – вывод записей с датой создания больше указанной даты
  - только dateEnd – вывод записей с датой создания меньше указанной даты
- **status** – ограничение по статусам, через запятую ['0' => 'Новое', '1' => 'Обработано', '2' => 'Отменено']

**Пример формирования запроса в PHP:**

<pre><code class="php">
  $params = [
 	"login" => "vladislav@isaler.ru",
	"apikey" => "aMgiCQyj8bCToNc47BZZYrRICoWSIl",
	// указываем метод
	"action"    => 'list',
	// страница, с учетом вывода 200 записей на страницу
	"offset"    => 0,
	// поле для упорядочивания записей
	"order"     => 'datum',
	// направление сортировки; new - по-умолчанию, old - сначала более старые
	"first"     => '',
	// ограничение по login пользователя, пользователь должен быть в подчинении у текущего
	"user"      => '',
	// даты создания
	"dateStart" => '2016-01-01',
	"dateEnd"   => '2018-06-01',
	//статус обращения
	"status"    => '1,2'
  ]

$urlparams = http_build_query($params);

</code></pre>

**Ответ:**

В поле “data” приходит список записей:

<pre><code class="json">
{
	"data":[ 
		{
			"ide":"59",
			"datum":"2018-03-27 10:05:21",
			"datum_do":"0000-00-00 00:00:00",
			"content":"",
			"clid":"6179",
			"client":"ПолоМарко",
			"pid":"2706",
			"person":"Марко Поло",
			"did":"0",
			"deal":null,
			"iduser":"22",
			"user":"Марусин Андрей Вениаминович",
			"idautor":"1",
			"autor":"Андреев Владислав",
			"status":"0",
			"statusName":"Новое",
			"products":[
				{"prid":"2354","title":"BizFAX E200 факс-сервер, 2 FXO, 1 FXS, 1 RJ45","kol":"5","price":"26724"},
				{"prid":"2316","title":"SIP-T26P SIP-телефон, 3 линии, BLF, PoE","kol":"10","price":"6460"}
			]
		} 
	]
}
</code></pre>


**Возможные ответы в случае ошибок:**

    400 – Не верный API key
    401 – Неизвестный пользователь
    402 – Неизвестный метод

<a id="info"></a>
### Запрос “info”

Запрос служит для вывода информации по id или uid обращения.

**Параметры запроса:**

- **id** – уникальный идентификатор записи
- **uid** – уникальный идентификатор записи из внешней системы

**Пример запроса:**

<pre><code class="php">
  $params = [
 	"login" => "vladislav@isaler.ru",
	"apikey" => "aMgiCQyj8bCToNc47BZZYrRICoWSIl",
	// указываем метод
	"action"   => 'info',
	// id записи
	"id"     => '56',
	// или uid записи
	"uid"      => '125'
  ]

$urlparams = http_build_query($params);

</code></pre>

**Ответ:**

В поле “data” приходит информация в виде строки в формате JSON и содержит следующую информацию:

- **id** - уникальный идентификатор записи
- **uid** - уникальный идентификатор записи из внешней системы, если есть
- **clid** - уникальный идентификатор записи клиента
- **pid** - уникальный идентификатор записи контакта
- **did** - уникальный идентификатор записи сделки, если Обращение преобразовано в сделку
- **datum** - дата-время создания записи
- **datum_do** - дата-время обработки Обращения
- **iduser** - идентификатор Ответственного
- **autor** - идентификатор Автора записи
- **content** - комментарий
- **status** - статус Обращения ['0' => 'Новое', '1' => 'Обработано', '2' => 'Отменено']
- **products** - спецификация
  - prid - уникальный идентификатор позиции из прайса
  - title -  название позиции
  - kol - количество
  - price - цена продажи

**Пример ответа:**

<pre><code class="json">
{"data": {
    "ide":"56",
    "uid":null,
    "clid":"6164",
    "pid":"2689",
    "did":"0",
    "datum":"2017-02-02 14:40:05",
    "datum_do":"0000-00-00 00:00:00",
    "iduser":"1",
    "autor":"1",
    "content":"",
    "status":"0",
    "products":[{
             "idp":"62",
             "ide":"56",
             "prid":"2315",
             "title":"SIP-T22P SIP-телефон, 3 линии, PoE",
             "kol":"3",
             "price":"4318",
            "identity":"1"
        },
        {
             "idp":"63",
             "ide":"56",
             "prid":"2354",
             "title":"BizFAX E200 факс-сервер, 2 FXO, 1 FXS, 1 RJ45",
             "kol":"1",
            "price":"26724"
        }]
    }
}
</code></pre>


Возможные ответы в случае ошибок:

    400 – Не верный API key
    401 – Неизвестный пользователь
    402 – Неизвестный метод

<a id="status"></a>
### Запрос “status”

Запрос позволяет обновить значение поля uid по id Обращения, а также его статус

**Параметры запроса:**

- **id** – уникальный идентификатор Обращения
- **uid** – уникальный идентификатор из внешней системы

**Пример формирования запроса в PHP:**

<pre><code class="php">
  $params = [
 	"login" => "vladislav@isaler.ru",
	"apikey" => "aMgiCQyj8bCToNc47BZZYrRICoWSIl",
	// указываем метод
	"action" => 'status',
	// id записи
	"id"     => '56',
	// uid записи для обновления
	"uid"    => '125',
	//статус обращения
	//"status" => '1'
  ]

$urlparams = http_build_query($params);

</code></pre>

**Ответ:**

<pre><code class="json">
{"result":"Успешно","data":"56"}
</code></pre>


**Возможные ответы в случае ошибок:**

    400 – Не верный API key
    401 – Неизвестный пользователь
    402 – Неизвестный метод
    404 – Обращение не найдено
