# История (history)

## Метод “history”

Метод позволяет управлять записями Напоминаний – добавлять, обновлять, удалять.

<pre><code class="html">URL для вызова - http(s)://crm_url/developer/v2/history?параметр=значение</code></pre>

Специфичные признаки, используемые далее:

- **cid** – уникальный идентификатор напоминания (в запросах идет как **id**)

```
* по умолчанию
```

<a id="fields"></a>
### Запрос “fields”

Запрос позволяет получить список доступных полей, хранящих информацию о клиенте в формате – «Имя поля» - «Расшифровка назначения» для формирования дальнейших запросов.

В список включены поля, активированные в Панели управления / Формы. Не активные поля игнорируются при обработке.

**Пример запроса:**

<pre><code class="php">
$params = [
    "login"  => "vladislav@isaler.ru",
    "apikey" => "aMgiCQyj8bCToNc47BZZYrRICoWSIl",
    "action" => "fields"
]

$urlparams = http_build_query($params);

</code></pre>

**Ответ:**

<pre><code class="json">
{
	"data":{
		"tid":"Идентификатор напоминания",
		"title":"Тема","des":"Агенда",
		"tip":"Тип напоминания",
		"active":"Признак: выполнено,активно",
		"priority":"Важность",
		"spees":"Срочность",
		"iduser":"Ответственный",
		"autor":"Автор напоминания",
		"clid":"ID клиента",
		"pid":"ID контакта",
		"did":"ID сделки"
	}
}
</code></pre>


**Возможные ответы в случае ошибок:**

    400 – Не верный API key
    401 – Неизвестный пользователь
    402 – Неизвестный метод

<a id="tips"></a>
### Запрос “tips”

Запрос позволяет получить список доступных типов дел и их цвета для формирования дальнейших запросов.

**Пример запроса:**

<pre><code class="php">
$params = [
    "login"  => "vladislav@isaler.ru",
    "apikey" => "aMgiCQyj8bCToNc47BZZYrRICoWSIl",
    "action" => "fields"
]

$urlparams = http_build_query($params);

</code></pre>

**Ответ:**

Парметр **filter** в ответе означает принадлежность активности:

1. **all** - относится и к Напоминаниям и к Истории активностей
2. **activ** - относится только к Истории активностей
3. **task** - относится только к Напоминаниям

<pre><code class="json">
{
	"data":[
		{"title":"Первичный звонок","color":"#009900","filter":"all"},
		{"title":"Факс","color":"#cc00cc","filter":"activ"},
		{"title":"Встреча","color":"#ffcc00","filter":"all"},
		{"title":"Задача","color":"#ff6600","filter":"all"},
		{"title":"Предложение","color":"#66ccff","filter":"activ"},
		{"title":"Событие","color":"#666699","filter":"activ"},
		{"title":"исх.Почта","color":"#cccc00","filter":"all"},
		{"title":"вх.Звонок","color":"#99cc00","filter":"all"},
		{"title":"вх.Почта","color":"#cc3300","filter":"all"},
		{"title":"Поздравление","color":"#009999","filter":"task"},
		{"title":"исх.2.Звонок","color":"#339966","filter":"all"},
		{"title":"Отправка КП","color":"#ff0000","filter":"all"},
		{"title":"Исходящий звонок КЦ","color":"#3498DB","filter":"activ"},
		{"title":"+ Сделка","color":"#339966","filter":"all"},
		{"title":"Вакансия","color":"#c67c00","filter":"all"},
		{"title":"Холодный звонок","color":"#99ccff","filter":"activ"}
	]
}
</code></pre>


**Возможные ответы в случае ошибок:**

    400 – Не верный API key
    401 – Неизвестный пользователь
    402 – Неизвестный метод

<a id="info"></a>
### Запрос “info”

Запрос позволяет получить информацию по напоминанию по его идентификатору - tid.

**Параметры запроса:**

- **id** – уникальный идентификатор записи напоминания

**Пример запроса:**

<pre><code class="php">
$params = [
    "login"  => "vladislav@isaler.ru",
    "apikey" => "aMgiCQyj8bCToNc47BZZYrRICoWSIl",
	// указываем метод
	"action" => 'info',
	"id"     => 1583
]

$urlparams = http_build_query($params);

</code></pre>

**Ответ:**

<pre><code class="json">
{
	"data":{
		"cid":17947,
		"datum":"2018-07-18 15:39:15",
		"content":"Работа проведена. Круто-чо!",
		"tip":"Событие",
		"iduser":"1",
		"user":"vladislav@isaler.ru",
		"clid":"1781",
		"client":"Сейлзмен Рус",
		"person":[
			{"pid":"2723","person":"Ларшин Олег"},
			{"pid":"2513","person":"Васильев Дмитрий"},
			{"pid":"2475","person":"Андреев Владислав Германович"}
		],
		"did":"784",
		"dogovor":"A25: Поставка телефонов"
	}
}
</code></pre>

**Примечание:**

1. Контакты возвращаются в виде массива


**Возможные ответы в случае ошибок:**

    400 – Не верный API key
    401 – Неизвестный пользователь
    402 – Неизвестный метод
    403 – Напоминание с указанным id не найдено в пределах аккаунта указанного пользователя
    404 – Не найдено
    405 – Отсутствуют параметры - id напоминания

<a id="list"></a>
### Запрос “list”

Запрос позволяет получить список напоминаний, доступных текущему сотруднику, в т.ч. с применением фильтров.

**Параметры запроса (не обязательные):**

- **offset** – страница вывода, с учетом того, что установлен лимит в 200 записей на страницу (по умолчанию offset = 0)
- **order** – поле, по которому будет производится сортировка списка (по умолчанию order = datum)
- **first** – направление сортировки (new – сначала новые, old – сначала старые ). (по умолчанию first = new)

**Фильтры (не обязательные):**

- **user** – ограничение по пользователю (указывается логин пользователя)
- **dateStart** – начальная дата напоминания (формат - YYYY-MM-DD)
- **dateEnd** – конечная дата напоминания (не обязательно, формат - YYYY-MM-DD)
  - только dateStart – вывод записей с датой выполнения больше указанной даты
  - только dateEnd – вывод записей с датой выполнения меньше указанной даты
- **word** – слово поиска по полям title, des, tip
- **tip** – поиск по определенному типу напоминания (title в ответе запроса tips)
- **clid** - фильтр по Клиенту
- **did** - фильтр по Сделке
- **pid** - фильтр по Контакту

**Примечание:**

1. По умолчанию выводятся напоминания на текущий день
2. Если не указан **user**, то будут выведены записи текущего пользователя и всех подчиненных

**Пример запроса:**

<pre><code class="php">
$params = [
    "login"     => "vladislav@isaler.ru",
    "apikey"    => "aMgiCQyj8bCToNc47BZZYrRICoWSIl",
    "action"    => 'list',
	// страница, с учетом вывода 200 записей на страницу
	"offset"    => 0,
	// поле для упорядочивания записей
	"order"     => 'datum',
	// направление сортировки; new - по-умолчанию, old - сначала более старые
	"first"     => '',
	// ограничение по login пользователя
	//"user"      => 'marand@omadaru.ru',
	// даты создания
	"dateStart" => '2018-07-01',
	"dateEnd"   => '2018-07-31',
	// фильтр по полям des, tip
	"word"      => '',
	//фильтр по типу активности
	"tip"       => '',
	//фильтр по клиенту
	"clid"      => 0,
	//фильтр по сделке
	"did"       => 784,
	//фильтр по контакту
	"pid"       => 0,
]

$urlparams = http_build_query($params);

</code></pre>

**Ответ:**

- Ответ аналогичен запросу **info**

**Возможные ответы в случае ошибок:**

    400 – Не верный API key
    401 – Неизвестный пользователь
    402 – Неизвестный метод


<a id="add"></a>
### Запрос “add”

Запрос позволяет добавить активность в базу CRM. При этом ответственным устанавливается сотрудник, логин которого использовался в запросе или указанный отдельно в параметре user

**Параметры запроса:**

- **user** – login пользователя
- **datum** => Дата в формате YYYY-MM-DD H:i:s (по умолчанию текущие дата и время)
- **content** => Содержимое записи истории
- **tip** => Тип активности
- **clid** => ID клиента
- **pid** => список ID контактов, в виде строки с разделителем ","
- **did** => ID сделки

**Примечание:**

1. при пустом поле **user** Ответственным будет назначен текущий пользователь (из запроса)
2. следующие параметры передаются в явном, текстовом виде:
   - **tip** – Тип активности (из справочника Типы напоминаний). Если не указано, то будет установлен как "Событие"
   - В случае отсутствия переданных значений в справочниках будут созданы новые записи

**Пример формирования запроса в PHP:**

<pre><code class="php">
$params = [
    "login"  => "vladislav@isaler.ru",
    "apikey" => "aMgiCQyj8bCToNc47BZZYrRICoWSIl",
    // указываем метод
	"action"  => 'add',
	// login пользователя
	"user"     => 'marand@omadaru.ru',
	"datum"   => '2018-07-18',
	"tip"     => 'Событие',
	"content" => 'Это описание 2',
	"clid"    => '1781',
	"pid"     => '2475,2723',
	"did"     => '784',
]

$urlparams = http_build_query($params);

</code></pre>

**Ответ:**

В поле “data” приходит id созданной записи

<pre><code class="json">
{"result":"Успешно","data":2502}
</code></pre>


**Возможные ответы в случае ошибок:**

    400 – Не верный API key
    401 – Неизвестный пользователь
    402 – Неизвестный метод
    405 – Отсутствуют параметры

<a id="add.list"></a>
### Запрос “add.list”

Запрос позволяет добавить массив активностей в базу CRM. При этом ответственным устанавливается сотрудник, логин которого использовался в запросе или указанный отдельно в параметре user

**Параметры запроса:**

- **user** – login пользователя
- **datum** => Дата в формате YYYY-MM-DD H:i:s (по умолчанию текущие дата и время)
- **content** => Содержимое записи истории
- **tip** => Тип активности
- **clid** => ID клиента
- **pid** => список ID контактов, в виде строки с разделителем ","
- **did** => ID сделки

**Примечание:**

1. параметры передаются для каждой записи отдельно в виде элемента массива
2. при пустом поле **user** Ответственным будет назначен текущий пользователь (из запроса)
3. следующие параметры передаются в явном, текстовом виде:
   - **tip** – Тип активности (из справочника Типы напоминаний). Если не указано, то будет установлен как "Событие"
   - В случае отсутствия переданных значений в справочниках будут созданы новые записи

**Пример формирования запроса в PHP:**

<pre><code class="php">
$params = [
    "login"  => "vladislav@isaler.ru",
    "apikey" => "aMgiCQyj8bCToNc47BZZYrRICoWSIl",
    // указываем метод
	"action" => 'add.list',
	// login пользователя, если в записи не указано
	"user"   => 'marand@omadaru.ru',
	//позиции спецификации
	"list"   => [
		[
			"user"    => 'marand@omadaru.ru',
			"datum"   => '2018-08-11 15:30:00',
			"content" => 'Это описание 100',
			"tip"     => 'Исх.звонок',
			"clid"    => '1781',
			"pid"     => '2475,2723',
			"did"     => '784'
		],
		[
			"datum"   => '2018-08-11 15:30:00',
			"content" => 'Это описание 200',
			"tip"     => 'Отправка КП',
			"clid"    => '1781',
			"pid"     => '2475,2723',
			"did"     => '784'
		]
	]
]

$urlparams = http_build_query($params);

</code></pre>

**Ответ:**

В поле “data” приходит id созданной записи

<pre><code class="json">
[
	{"result":"Успешно","data":17952},
	{"result":"Успешно","data":17953}
]
</code></pre>


**Возможные ответы в случае ошибок:**

    400 – Не верный API key
    401 – Неизвестный пользователь
    402 – Неизвестный метод
    405 – Отсутствуют параметры

<a id="delete"></a>
### Запрос “delete”

Запрос позволяет удалить напоминание по его id.

**Параметры запроса:**

- **id** – уникальный идентификатор активности (обязательное поле)

**Пример запроса:**

<pre><code class="php">
$params = [
    "login"  => "vladislav@isaler.ru",
    "apikey" => "aMgiCQyj8bCToNc47BZZYrRICoWSIl",
    "action" => 'delete',
	"id"     => 17953
]

$urlparams = http_build_query($params);

</code></pre>

**Ответ:**

<pre><code class="json">
{"result":"Успешно"}
</code></pre>


**Возможные ответы в случае ошибок:**

    400 – Не верный API key
    401 – Неизвестный пользователь
    402 – Неизвестный метод
    403 – Напоминание не найдено
    405 – Отсутствуют параметры - tid напоминания

