# Сотрудники (user)

## Метод “user”

Метод позволяет работать с записями сотрудников.

<pre><code class="html">URL для вызова - http(s)://crm_url/developer/v2/user?параметр=значение</code></pre>

**Пример запроса:**

http(s)://crm_url/developer/v1/user?login=vladislav@isaler.ru&apikey=aMgiCQyj8bCToNc47BZZYrRICoWSIl&action=users

Возможные ответы в случае ошибок:

    400 – Не верный API key
    401 – Неизвестный пользователь
    402 – Неизвестный метод

<a id="user"></a>
### Запрос “user”

Запрос позволяет получить расширенню информацию о Пользователе аккаунта

**Параметры запроса:**

- **user** – Логин пользователя
- **uid** – уникальный идентификатор, связанный с внешней ИС или
- **iduser** - уникальный идентификатор в CRM

Можно указать либо uid (он должен быть добавлен в систему), либо iduser требуемого пользователя. Первоначально проверяется uid.

<pre><code class="php">

$params = [
	"login"  => "vladislav@isaler.ru",
	"apikey" => "aMgiCQyj8bCToNc47BZZYrRICoWSIl",
	// указываем метод
	"action" => 'user',
	//uid пользователя, если известен
	"uid"    => '',
	// ограничение по login пользователя, пользователь должен быть в подчинении у текущего
	"user"   => 'vladislav@isaler.ru',
];

$urlparams = http_build_query($params);

</code></pre>

**Ответ:**

Полученный ответ содержит данные:

- **iduser** - уникальный идентификатор в CRM
- **uid** – уникальный идентификатор, связанный с внешней ИС
- **oid** – уникальный идентификатор отдела, связанный с внешней ИС
- **title** - имя Пользователя
- **login** – логин Пользователя
- **active** – Статус пользователя (yes – активен, no – заблокирован)
- **tip** - роль в системе
- **user_post** - должность
- **email** - email пользователя (для пользователей нашего сервиса совпадает с логином)
- **phone** - телефон
- **phone_in** - внутренний (добавочный) номер (для sip-аккаунтов)
- **mob** - мобильный телефон

<pre><code class="json">
{
	"data":{
		"iduser":"1",
		"uid":"DIR100",
		"oid":"OAP",
		"title":"Андреев Владислав",
		"login":"vladislav@isaler.ru",
		"active":"yes",
		"tip":"Руководитель подразделения",
		"user_post":"Руководитель проекта",
		"email":"vladislav@isaler.ru",
		"phone":"7(495)373-07-63",
		"phone_in":"771",
		"mob":"+79223289467"
	}
}
</code></pre>

<a id="user.list"></a>
### Запрос “user.list”

Запрос позволяет получить список Пользователей аккаунта

<pre><code class="php">

$params = [
	"login"  => "vladislav@isaler.ru",
	"apikey" => "aMgiCQyj8bCToNc47BZZYrRICoWSIl",
	// указываем метод
	"action" => 'user.list'
];

$urlparams = http_build_query($params);

</code></pre>

**Ответ:**

Полученный ответ содержит данные по каждому сотруднику:

- **iduser** - уникальный идентификатор в CRM
- **uid** – уникальный идентификатор, связанный с внешней ИС
- **oid** – уникальный идентификатор отдела, связанный с внешней ИС
- **title** – Имя пользователя
- **login** – Логин пользователя
- **email** – Email пользователя
- **active** – Статус пользователя (yes – активен, no – заблокирован)


<pre><code class="json">
{
	"data":[
		{"iduser":"1","uid":"DIR100","oid":"OAP","title":"Андреев Владислав","login":"vladislav@isaler.ru","email":"vladislav@isaler.ru","active":"yes"},
		{"iduser":"3","uid":"","oid":"OAP","title":"Хайрулин Виктор","login":"viktor@isaler.ru","email":"viktor@isaler.ru","active":"yes"},
		{"iduser":"13","uid":"","oid":"OPP","title":"Потапов Андрей","login":"demo","email":"venik@gmail.com","active":"yes"},
		{"iduser":"14","uid":"","oid":"OPP","title":"Михайлов Захар","login":"zahar","email":"zahar@isaler.ru","active":"yes"},
		{"iduser":"15","uid":"","oid":"OPP","title":"Ющенко Андрей","login":"yushenko","email":"yuschenko@isaler.ru","active":"yes"}
	]
}
</code></pre>

<a id="user.add"></a>
### Запрос “user.add”

Запрос позволяет добавить нового Пользователя

**Параметры запроса:**

- **uid** – уникальный идентификатор, связанный с внешней ИС
- **user** – Логин пользователя (для SalesMan Online это email)
- **password** - Пароль пользователя
- **title** – Имя пользователя
- **mid** - Идентификатор (uid) руководителя Пользователя в вашей системе (должен совпадать с идентификатором в CRM)
- **otdel** - Идентификатор (oid) отдела во внешней системе
- **tip** - роль в системе (Руководитель организации, Руководитель с доступом, Руководитель подразделения, Руководитель отдела, Менеджер продаж, Поддержка продаж, Специалист, Администратор)
- **user_post** - должность
- **email** - email пользователя (для пользователей нашего сервиса совпадает с логином)
- **phone** - телефон
- **phone_in** - внутренний телефон (для sip-аккаунтов)
- **mob** - мобильный телефон
- **boss** - логин руководителя

<pre><code class="php">

$params = [
	"login"  => "vladislav@isaler.ru",
	"apikey" => "aMgiCQyj8bCToNc47BZZYrRICoWSIl",
	// указываем метод
	"action" => 'user.add',
	//uid пользователя, если известен
	"uid"    => '',
	// ограничение по login пользователя, пользователь должен быть в подчинении у текущего
	"user"   => 'ivan.petrov@isaler.ru',
	//пароль в явном виде
	"password" => 'PassW0Rd!2',
	//ФИО
	"title" => 'Иван Петров',
	//роль в системе (Руководитель организации, Руководитель с доступом, Руководитель подразделения, Руководитель отдела, Менеджер продаж, Поддержка продаж, Специалист, Администратор)
	"tip" => "",
	//должность
	"user_post" => "Менеджер продаж",
	//день рождения
	"bday" => "1965-02-02",
	//телефон
	"phone" => "",
	//добавочный номер
	"phone_in" => "",
	//мобильный номер
	"mob" => "",
	//email пользоваптеля
	"email" => 'ivan.petrov@isaler.ru',
	//логин руководителя
	"boss" => "vladislav@isaler.ru",
	//отдел
	"otdel" => "OAP"
];

$urlparams = http_build_query($params);

</code></pre>

**Ответ:**

В случае успешного запроса вернется iduser в поле data

<pre><code class="json">
{"result":"Success","data":75}
</code></pre>

**Возможные ответы в случае ошибок:**

    400 – Не верный API key
    401 – Неизвестный пользователь
    402 – Неизвестный метод
    406 - Такой логин уже существует
    500 - Внутренняя ошибка
