# Обращения (entry)

## Метод “entry”

Метод позволяет запрашивать информацию по Обращениям

URL для вызова:
```http
http(s)://{{baseurl}}/developer/v3/akt/entry?параметр=значение
```


<a id="list"></a>
### Запрос “list”

Запрос позволяет получить список сделок, доступных текущему сотруднику, в т.ч. с применением фильтров.

**Параметры запроса (не обязательные):**

- **offset** – страница вывода, с учетом того, что установлен лимит в 200 записей на страницу (по умолчанию offset = 0)
- **order** – поле, по которому будет производится сортировка списка (по умолчанию order = datum)
- **first** – направление сортировки (new – сначала новые, old – сначала старые). (по умолчанию first = new)

**Фильтры (не обязательные):**

- **user** – ограничение по пользователю (указывается логин пользователя)
- **dateStart** – начальная дата создания записи (формат - YYYY-MM-DD)
- **dateEnd** – конечная дата создания записи (не обязательно, формат - YYYY-MM-DD)
  - только dateStart – вывод записей с датой создания больше указанной даты
  - только dateEnd – вывод записей с датой создания меньше указанной даты
- **status** – ограничение по статусам, через запятую ['0' => 'Новое', '1' => 'Обработано', '2' => 'Отменено']

**Пример запроса:**

```http request
GET http://{{baseurl}}/developer/v3/entry/list
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru

{
	"dateStart": "2017-06-01",
	"dateEnd": "2018-06-30",
	"status": "1,2"
}
```

**Ответ:**

В поле “data” приходит список записей:

```json
{
	"data": [
		{
			"ide": 59,
			"datum": "2018-03-27 12:05:21",
			"datum_do": "2018-08-17 17:57:00",
			"content": "fdgfdgd",
			"clid": 0,
			"client": null,
			"pid": 2706,
			"person": "Марко Поло",
			"did": 0,
			"deal": null,
			"iduser": 22,
			"user": "Марусин Андрей Вениаминович",
			"idautor": 1,
			"autor": "Андреев Владислав",
			"status": 2,
			"statusName": "Отменено",
			"products": [
				{
					"prid": 2354,
					"title": "BizFAX E200 факс-сервер, 2 FXO, 1 FXS, 1 RJ45",
					"kol": 5,
					"price": 26724
				},
				{
					"prid": 2316,
					"title": "SIP-T26P SIP-телефон, 3 линии, BLF, PoE",
					"kol": 10,
					"price": 6460
				}
			]
		}
	]
}
```

<a id="info"></a>
### Запрос “info”

Запрос служит для вывода информации по id обращения.

**Параметры запроса:**

- **id** – уникальный идентификатор записи

**Пример запроса:**

```http request
GET http://{{baseurl}}/developer/v3/entry/info
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru

{
    "id": 59
}
```

**Ответ:**

В поле “data” приходит информация в виде строки в формате JSON и содержит следующую информацию:

- **id** - уникальный идентификатор записи
- **uid** - уникальный идентификатор записи из внешней системы, если есть
- **clid** - уникальный идентификатор записи клиента
- **pid** - уникальный идентификатор записи контакта
- **did** - уникальный идентификатор записи сделки, если Обращение преобразовано в сделку
- **datum** - дата-время создания записи
- **datum_do** - дата-время обработки Обращения
- **iduser** - идентификатор Ответственного
- **autor** - идентификатор Автора записи
- **content** - комментарий
- **status** - статус Обращения ['0' => 'Новое', '1' => 'Обработано', '2' => 'Отменено']
- **products** - спецификация
  - prid - уникальный идентификатор позиции из прайса
  - title -  название позиции
  - kol - количество
  - price - цена продажи

**Пример ответа:**

```json
{
	"data": {
		"ide": 59,
		"uid": 0,
		"clid": 0,
		"pid": 2706,
		"did": 0,
		"datum": "2018-03-27 04:05:21",
		"datum_do": "2018-08-17 09:57:00",
		"iduser": 22,
		"autor": 1,
		"content": "fdgfdgd",
		"status": 2,
		"identity": 1,
		"products": [
			{
				"idp": 67,
				"ide": 59,
				"prid": 2354,
				"title": "BizFAX E200 факс-сервер, 2 FXO, 1 FXS, 1 RJ45",
				"kol": 5,
				"price": 26724,
				"identity": 1
			},
			{
				"idp": 72,
				"ide": 59,
				"prid": 2316,
				"title": "SIP-T26P SIP-телефон, 3 линии, BLF, PoE",
				"kol": 10,
				"price": 6460,
				"identity": 1
			}
		]
	}
}
```


Возможные ответы в случае ошибок:

    400 – Не верный API key
    401 – Неизвестный пользователь
    402 – Неизвестный метод

<a id="status"></a>
### Запрос “status”

Запрос позволяет обновить статус обращения

**Параметры запроса:**

- **id** – уникальный идентификатор Обращения
- **uid** – уникальный идентификатор из внешней системы

**Пример формирования запроса в PHP:**

```http request
POST http://{{baseurl}}/developer/v3/entry/status
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru

{
	"id": 59,
    "status": 1
}
```

**Ответ:**

```json
{
	"result": "Успешно",
	"data": 59
}
```

**Возможные ответы в случае ошибок:**

    404 – Обращение не найдено
