# История (history)

## Метод “history”

Метод позволяет управлять записями Напоминаний – добавлять, обновлять, удалять.

URL для вызова:
```http
http(s)://{{baseurl}}/developer/v3/history/запрос?параметр=значение
```

Специфичные признаки, используемые далее:

- **cid** – уникальный идентификатор напоминания (в запросах идет как **id**)


<a id="fields"></a>
### Запрос “fields”

Запрос позволяет получить список доступных полей, хранящих информацию о клиенте в формате – «Имя поля» - «Расшифровка назначения» для формирования дальнейших запросов.

В список включены поля, активированные в Панели управления / Формы. Не активные поля игнорируются при обработке.

**Пример запроса:**

```http request
GET http://{{baseurl}}/developer/v3/history/fields
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru
```

**Ответ:**

```json
{
	"data": {
		"cid": "Идентификатор записи",
		"datum": "Дата",
		"des": "Содержание",
		"tip": "Тип активности",
		"iduser": "Ответственный",
		"clid": "ID клиента",
		"pid": "ID контакта (массив)",
		"did": "ID сделки"
	}
}
```


<a id="tips"></a>
### Запрос “tips”

Запрос позволяет получить список доступных типов дел и их цвета для формирования дальнейших запросов.

**Пример запроса:**

```http request
GET http://{{baseurl}}/developer/v3/history/tips
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru
```

**Ответ:**

Парметр **filter** в ответе означает принадлежность активности:

1. **all** - относится и к Напоминаниям и к Истории активностей
2. **activ** - относится только к Истории активностей
3. **task** - относится только к Напоминаниям

```json
{
	"data": [
		{
			"title": "Первичный звонок",
			"color": "#92cddc",
			"filter": "all"
		},
		{
			"title": "Факс",
			"color": "#cc00cc",
			"filter": "activ"
		},
		{
			"title": "Встреча",
			"color": "#31859b",
			"filter": "all"
		},
		{
			"title": "Задача",
			"color": "#ff6600",
			"filter": "all"
		},
		{
			"title": "Предложение",
			"color": "#66ccff",
			"filter": "activ"
		},
		{
			"title": "Событие",
			"color": "#666699",
			"filter": "activ"
		},
		{
			"title": "исх.Почта",
			"color": "#cccc00",
			"filter": "all"
		},
		{
			"title": "вх.Звонок",
			"color": "#99cc00",
			"filter": "all"
		},
		{
			"title": "вх.Почта",
			"color": "#cc3300",
			"filter": "all"
		},
		{
			"title": "Поздравление",
			"color": "#009999",
			"filter": "task"
		},
		{
			"title": "исх.2.Звонок",
			"color": "#339966",
			"filter": "all"
		},
		{
			"title": "Отправка КП",
			"color": "#ff0000",
			"filter": "all"
		},
		{
			"title": "Исходящий звонок КЦ",
			"color": "#3498DB",
			"filter": "activ"
		},
		{
			"title": "+ Сделка",
			"color": "#c09100",
			"filter": "all"
		},
		{
			"title": "++Вакансия",
			"color": "#c67c00",
			"filter": "all"
		},
		{
			"title": "Холодный звонок",
			"color": "#99ccff",
			"filter": "activ"
		}
	]
}
```

<a id="list"></a>
### Запрос “list”

Запрос позволяет получить список напоминаний, доступных текущему сотруднику, в т.ч. с применением фильтров.

**Параметры запроса (не обязательные):**

- **offset** – страница вывода, с учетом того, что установлен лимит в 200 записей на страницу (по умолчанию offset = 0)
- **order** – поле, по которому будет производится сортировка списка (по умолчанию order = datum)
- **first** – направление сортировки (new – сначала новые, old – сначала старые ). (по умолчанию first = new)

**Фильтры (не обязательные):**

- **user** – ограничение по пользователю (указывается логин пользователя)
- **dateStart** – начальная дата напоминания (формат - YYYY-MM-DD)
- **dateEnd** – конечная дата напоминания (не обязательно, формат - YYYY-MM-DD)
    - только dateStart – вывод записей с датой выполнения больше указанной даты
    - только dateEnd – вывод записей с датой выполнения меньше указанной даты
- **word** – слово поиска по полям title, des, tip
- **tip** – поиск по определенному типу напоминания (title в ответе запроса tips)
- **clid** - фильтр по Клиенту
- **did** - фильтр по Сделке
- **pid** - фильтр по Контакту

**Примечание:**

1. По умолчанию выводятся напоминания на текущий день
2. Если не указан **user**, то будут выведены записи текущего пользователя и всех подчиненных

**Пример запроса:**

```http request
GET http://{{baseurl}}/developer/v3/history/list
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru

{
	"order": "datum",
	"dateStart": "2012-12-18",
	"dateEnd": "2012-12-18"
}
```

**Ответ:**

- Ответ аналогичен запросу **info**


<a id="info"></a>
### Запрос “info”

Запрос позволяет получить информацию по напоминанию по его идентификатору - cid.

**Параметры запроса:**

- **cid** – уникальный идентификатор записи напоминания

**Пример запроса:**

```http request
GET http://{{baseurl}}/developer/v3/history/info
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru

{
	"cid": 777
}
```

**Ответ:**

```json
{
	"data": {
		"cid": 777,
		"datum": "2012-12-18 10:46:58",
		"content": "Отправлено КП;",
		"tip": "Предложение",
		"iduser": 1,
		"user": "vladislav@isaler.ru",
		"clid": 102,
		"client": "КАМКАБЕЛЬ, ОАО",
		"person": [
			{
				"pid": 168,
				"person": "Васильев Дмитрий"
			}
		],
		"did": 0,
		"dogovor": ""
	}
}
```

**Примечание:**

1. Контакты возвращаются в виде массива person


**Возможные ответы в случае ошибок:**

    403 – Запись с указанным cid не найдено в пределах аккаунта указанного пользователя
    404 – Не найдено
    405 – Отсутствуют параметры - сid записи


<a id="add"></a>
### Запрос “add”

Запрос позволяет добавить активность в базу CRM. При этом ответственным устанавливается сотрудник, логин которого использовался в запросе или указанный отдельно в параметре user

**Параметры запроса:**

- **user** – login пользователя
- **datum** – Дата в формате YYYY-MM-DD H:i:s (по умолчанию текущие дата и время)
- **content** – Содержимое записи истории
- **tip** – Тип активности
- **clid** – ID клиента
- **pid** – список ID контактов, в виде строки с разделителем ","
- **did** – ID сделки

**Примечание:**

1. при пустом поле **user** Ответственным будет назначен текущий пользователь (из запроса)
2. следующие параметры передаются в явном, текстовом виде:
   - **tip** – Тип активности (из справочника Типы напоминаний). Если не указано, то будет установлен как "Событие"
   - В случае отсутствия переданных значений в справочниках будут созданы новые записи

**Пример запроса:**

```http request
POST http://{{baseurl}}/developer/v3/history/add
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru

{
	"user": "marand@omadaru.ru",
	"datum": "2024-06-25 14:25",
	"tip": "Предложение",
	"content": "Отправлено КП",
	"clid": 1781,
	"pid": "2475,2723",
	"did": 784
}
```

**Ответ:**

В поле “data” приходит id созданной записи

```json
{
	"result": "Успешно",
	"data": 25353
}
```

**Возможные ответы в случае ошибок:**

    405 – Отсутствуют параметры


<a id="add.list"></a>
### Запрос “add.list”

Запрос позволяет добавить массив активностей в базу CRM. При этом ответственным устанавливается сотрудник, логин которого использовался в запросе или указанный отдельно в параметре user

**Параметры запроса:**

- **user** – login пользователя
- **datum** - Дата в формате YYYY-MM-DD H:i:s (по умолчанию текущие дата и время)
- **content** - Содержимое записи истории
- **tip** - Тип активности
- **clid** - ID клиента
- **pid** - список ID контактов, в виде строки с разделителем ","
- **did** - ID сделки

**Примечание:**

1. параметры передаются для каждой записи отдельно в виде элемента массива list
2. при пустом поле **user** Ответственным будет назначен текущий пользователь (из запроса)
3. следующие параметры передаются в явном, текстовом виде:
   - **tip** – Тип активности (из справочника Типы напоминаний). Если не указано, то будет установлен как "Событие"
   - В случае отсутствия переданных значений в справочниках будут созданы новые записи

**Пример запроса:**

```http request
POST http://{{baseurl}}/developer/v3/history/add.list
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru

{
	"list": [
		{
			"user": "marand@omadaru.ru",
			"datum": "2024-06-25 14:20",
			"tip": "Исх.звонок",
			"content": "Это описание 100",
			"clid": 1781,
			"pid": "2475,2723",
			"did": 779
		},
		{
			"user": "marand@omadaru.ru",
			"datum": "2024-06-25 14:25",
			"tip": "Отправка КП",
			"content": "Это описание 101",
			"clid": 1781,
			"pid": "2475,2723",
			"did": 779
		}
	]
}
```

**Ответ:**

В поле “data” приходит id созданной записи

```json
[
	{
		"result": "Успешно",
		"data": 25354
	},
	{
		"result": "Успешно",
		"data": 25355
	}
]
```

**Возможные ответы в случае ошибок:**

    405 – Отсутствуют параметры


<a id="delete"></a>
### Запрос “delete”

Запрос позволяет удалить напоминание по его id.

**Параметры запроса:**

- **id** – уникальный идентификатор активности (обязательное поле)

**Пример запроса:**

```http request
POST http://{{baseurl}}/developer/v3/history/delete
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru

{
	"cid": 25355
}
```

**Ответ:**

```json
{
	"result": "Успешно"
}
```

**Возможные ответы в случае ошибок:**

    403 – Запись не найдена
    405 – Отсутствуют параметры - tid напоминания

