# Счета (invoice)

## Метод “invoice”

Метод позволяет управлять записями Сделок – добавлять, обновлять, удалять.

URL для вызова:
```http
http(s)://{{baseurl}}/developer/v3/invoice/запрос?параметр=значение
```


### Запрос “list” <a id="list"></a>

Запрос позволяет получить список счетов, доступных текущему сотруднику, в т.ч. с применением фильтров.

**Параметры запроса (не обязательные):**

- int **offset** – страница вывода, с учетом того, что установлен лимит в 200 записей на страницу (по умолчанию offset = 0)
- string **order** – поле, по которому будет производится сортировка списка (по умолчанию order = datePlan) (формат - YYYY-MM-DD)
  - dateCreate - по дате создания
  - datePlan - по плановой дате
  - dateFact - по дате оплаты
  - invoice - по номеру счета
  - summa - по сумме счета
- string **first** – направление сортировки (new – сначала новые, old – сначала старые). (по умолчанию first = new)

**Фильтры (не обязательные):**

- string **user** – ограничение по пользователю (указывается логин пользователя)
- int **iduser** – ограничение по пользователю (указывается ID пользователя)
- int **clid** - идентификатор клиента
- int **did** - идентификатор сделки
- по датам - dateCreate, datePlan, dateFact одинаково (для обозначения начала и завершения добавляются Start и End), например:
  - date **dateCreateStart** – начальная дата создания записи (формат - YYYY-MM-DD)
  - date **dateCreateEnd** – конечная дата создания записи (не обязательно, формат - YYYY-MM-DD)
      - только *Start – вывод записей с датой больше указанной даты
      - только *End – вывод записей с датой меньше указанной даты
- string **word** – слово поиска по номеру счета, названию сделки, названию клиента
- string **client** – слово поиска только по названию клиента
- bool **active** -  (true / false – по умолчанию) выводить только оплаченные или не оплаченные счета
- string **bankinfo** - для включения банковских реквизитов

**Пример формирования запроса:**

```http request
GET http://crm_url/developer/v3/invoice/list
Content-Type: application/json
apikey: t1xdeOwWSIqgDol70CkRdK3WD4N4cm
login: vladislav@isaler.ru

{
	"order": "date_create",
	"offset": 1,
	"datePlanStart": "2014-01-01",
	"datePlanEnd": "2014-01-02"
}
```

**Ответ:**

В поле “**data**” приходит список записей, в поле "**count**" - приходит общее количество записей в выборке

- **data** - список записей
- **params** - доп.данные
	- page - текущая страница
	- pageall - общее кол-во страниц
- **count** - общее количество записей в выборке

```json
{
	"data": [
		{
			"id": 638,
			"crid": 638,
			"date_create": "2015-07-10 16:26:03",
			"date_createf": "10.07.15",
			"date_plan": "2014-01-01",
			"date_planf": "01.01.2014",
			"date_fact": "2015-07-10",
			"date_factf": "10.07.2015",
			"contract": "",
			"invoice": "21",
			"summaf": "200 000,00",
			"summa": 200000,
			"color": null,
			"ddo": true,
			"warning": null,
			"do": true,
			"cando": null,
			"view": 1,
			"clid": 1404,
			"client": "Дорос, ООО",
			"pid": 0,
			"person": null,
			"did": 641,
			"deal": "Тестовая 4",
			"isclose": null,
			"count": "1 часть",
			"dole": null,
			"company": "ООО ”БИГСЕЙЛЗРУС”",
			"mc": 2,
			"deltaf": "10,00",
			"delta": 10
		}
	],
	"params": {
		"page": 1,
		"pageall": 1,
		"ord": "datum",
		"tuda": "DESC",
		"valuta": "р.",
		"count": 1
	},
	"count": 1
}
```

### Запрос “info” <a id="info"></a>

Запрос позволяет получить информацию по Счету по его идентификатору - id.

**Параметры запроса:**

- int **id** – уникальный идентификатор записи
- bool **dealinfo** - нужно ли возвращать полную информацию о сделке (true / false – по умолчанию)

**Пример запроса:**

```http request
GET http://crm_url/developer/v3/invoice/info
Content-Type: application/json
apikey: t1xdeOwWSIqgDol70CkRdK3WD4N4cm
login: vladislav@isaler.ru

{
	"id": 1018,
	"dealinfo": true
}
```

**Ответ:**

Ответ содержит следующие специфичные данные:

- **crid** - Идентификатор Счета
- **invoice** - Номер Счета
- **datum** - Дата Счета
- **datum_credit** - Плановая дата оплаты Счета
- **invoice_date** - Фактическая дата оплаты Счета
- **contractnum** - Номер договора
- **summa** - Сумма Счета
- **nds** - Сумма Налога
- **clid** - ID клиента
- **did** - ID сделки
- **tip** - Тип Счета
- **rs** - ID расчетного счета
- **signer** - id подписанта компании
- **iduser** - ID ответственного по счету
- **do** - Статус оплаты ("on" = оплачено)
- **suffix** - html-кодированный текст суффикса счета
- **template** - шаблон Счета
	- **id** - ID шаблона
	- **title** - название шаблона
	- **file** - файл шаблона
	- **typeid** - ID типа документа
- **deal** - полная информация о сделке, включая Заказчика, Плательщика, Спецификацию и пр.

```json
{
	"data": {
		"crid": 1018,
		"invoice": "266",
		"datum": "2023-12-20 21:40:00",
		"datum_credit": "2023-12-25",
		"invoice_date": null,
		"contractnum": null,
		"summa": 243000,
		"nds": 0,
		"clid": 1781,
		"pid": 0,
		"did": 972,
		"tip": "Счет-договор",
		"rs": 5,
		"iduser": 22,
		"do": "no",
		"suffix": "<p>&nbsp;<\/p>\r\n\r\n<p><b>ВНИМАНИЕ! СЧЕТ ДЕЙСТВИТЕЛЕН В ТЕЧЕНИЕ 5 ДНЕЙ С ДАТЫ СОЗДАНИЯ.<\/b><\/p>\r\n\r\n<p align=\"justify\">Стороны договорились, что стоимость услуг оплачивается Покупателем в размере 100 (сто) % от стоимости, указанной в вышеприведённой таблице. Датой оплаты счета считается дата списания денежных средств с расчетного счета Покупателя.<\/p>\r\n\r\n<p>Настоящий счет является договором-офертой в соответствии со ст.435 ГК РФ. Настоящая оферта действительна и ее акцепт возможен <b>в течение 5 (пяти) <\/b>рабочих дней с момента выставления счета. При осуществлении оплаты по окончании срока действия оферты, Поставщик вправе потребовать доплаты, при условии изменения текущей розничной стоимости товара, либо не принимать оплату. Оплата по настоящему счёту является заключением договора об оказании услуг, при условии принятия ее поставщиком, в том числе по истечении срока, установленного для акцепта данной оферты.<\/p>\r\n\r\n<p>&nbsp;<\/p>\r\n\r\n<p><b>Датой окончания предоставления услуги считается дата подписания обеими Сторонами акта приёма-передачи работ. Настоящий счёт-договор вступает в силу с момента его оплаты Покупателем и действует до момента исполнения всех обязательств по нему.<\/b><\/p>\r\n",
		"valuta": "р.",
		"template": [
			{
				"id": 155,
				"title": "Счет с QRcode",
				"file": "invoice_qr.tpl",
				"typeid": 94
			}
		],
		"idcurrency": 0,
		"currency": [],
		"idcourse": 0,
		"course": [],
		"signer": null
	}
}
```

**Возможные ответы в случае ошибок:**

    400 – Не верный API key
    401 – Неизвестный пользователь
    402 – Неизвестный метод
    404 – Документ не найден
    405 – Отсутствуют параметры - id Счета

### Запрос “add” <a id="add"></a>

Запрос позволяет добавить к сделке новый счет

**Параметры запроса:**

- string **uid** – уникальный идентификатор сделки во внешней системе ИЛИ
- int **did** – уникальный идентификатор сделки в CRM
- string **user** – логин Ответственного, если квота по счету идет сотруднику, не ответственному за сделку ИЛИ
- int **iduser** – ID пользователя (указывается ID пользователя)
- date **date** – дата счета (текущая - по умолчанию) (формат - YYYY-MM-DD)
- date **date.plan** – ожидаемая дата оплаты счета (+5 дней - по умолчанию) (формат - YYYY-MM-DD)
- float **summa** – сумма счета (если не указана - берем сумму Сделки)
- string **invoice** - номер счета, если пусто, то будем генерировать очередной из системы
- string **contract** - номер договора, если пусто, то смотрим Договор, прикрепленный к сделке
- int **rs** - идентификатор расчетного счета (список можно получить из справочника) - если пусто, то берем р/с компании, от которой ведется сделка с признаком "по-умолчанию"
- int **signer** - id подписанта компании
- float **nds** - размер НДС в абсолютных цифрах (не обязательно при наличии спецификации)
- string **tip** - тип счета 
  - ipre - Предварительная оплата, 
  - iafter - Окончательная оплата, 
  - ispeka - По спецификации, 
  - icontract - По договору, 
  - ioffer - Счет-договор
- **template** - файл шаблона (см. запрос "templates") или
- **templateID** - ID шаблона (см. запрос "templates")
- string **newstep** - значение этапа, на который нужно будет передвинуть сделку

**Если нужно зафиксировать уже оплаченный счет, то включаем следующие параметры:**

- **do** - признак оплаченного счета = yes
- **date.do** - дата оплаты, если пусто - текущая дата

> ### Важно
> 
> - Допускается указание либо **did**, либо **uid** сделки
> - Перед выставлением счета (если счет не к договору и не является предоплатным/постоплатным) необходимо добавить спецификацию - позиции с помощью запроса /deal/update
> - в случае **do = yes** счет просто отмечается оплаченным без каких-либо манипуляций с суммами на счете

**Пример запроса:**

```http request
POST http://crm_url/developer/v3/invoice/add
Content-Type: application/json
apikey: t1xdeOwWSIqgDol70CkRdK3WD4N4cm
login: vladislav@isaler.ru

{
	"did": 962,
	"tip": "Счет-договор",
	"invoice": "auto",
	"newstep": "60"
}
```

**Ответ:**

В поле “**result**“ приходит ответ обработчика, в поле “data” приходит:

- **id** - идентификатор записи счета в CRM
- **invoice** - номер счета

```json
{
	"result": "Успешно;Счет добавлен в платежи",
	"data": {
		"id": 1023,
		"invoice": "271"
	}
}
```

**Возможные ответы в случае ошибок:**

    403 – Сделка с указанным did не найдена в пределах аккаунта указанного пользователя
    406 – Уже выставлены все возможные счета

### Запрос “do” <a id="do"></a>

Запрос позволяет отметить выставленный счет оплаченным (с внесением суммы на указанный расчетный счет модуля Бюжет)

**Параметры запроса:**

- string **invoice** - номер счета
- int **id** - идентификатор записи счета (если номер не уникален или не указан)
- date **date.do** - дата оплаты, если пусто - текущая дата (формат - YYYY-MM-DD)
- float **summa** - сумма оплаты, если пусто, то принимается полная сумма счета; может быть меньше суммы счета - в таком случае будет создан счет на недостающую сумму
- string **newstep** - значение этапа, на который нужно будет передвинуть сделку

Наличие параметра "invoice" ИЛИ "id" обязательно для поиска счета в базе. Получить список счетов с "id" и "invoice" можно с помощью метода **info**

**Пример запроса:**

```http request
POST http://crm_url/developer/v3/invoice/do
Content-Type: application/json
apikey: t1xdeOwWSIqgDol70CkRdK3WD4N4cm
login: vladislav@isaler.ru

{
	"id": 1024,
	"newstep": "80"
}
```

**Ответ:**

В поле “result“ приходит ответ обработчика, в поле “data” приходит расширенный отчет обработки.

```json
{
	"result": "Успешно",
	"data": "Внесена оплата по графику 3 675 492,00 р.<br>На р\/с Основной расчетный счет внесена оплата в размере 3 675 492,00 р."
}
```

> ### Примечание
> 
> - Если внесена частичная оплата, то в ответе будет присутствовать параметр **newdata** - id дополнительного счета (с тем же номером)

```json
{
	"result": "Успешно",
	"data": "Внесена оплата по графику 3 000 000,00 р.<br>Поступившая оплата отличается от планируемой.<br>Добавлен дополнительный платеж 675 492,00 р.<br>На р\/с Основной расчетный счет внесена оплата в размере 3 000 000,00 р.",
	"newdata": 1025
}
```

**Возможные ответы в случае ошибок:**

    403 – Счет по ID или Номеру не найден

### Запрос “express” <a id="express"></a>

Является запросом, аналогичным add. Позволяет внести оплату с предварительным созданием счета. В случае неполной оплаты будет создана дополнительная запись в графике платежей на недостающую сумму.

**Параметры запроса:**

- string **uid** – уникальный идентификатор сделки во внешней системе ИЛИ
- int **did** – уникальный идентификатор сделки в CRM
- string **user** – логин Ответственного ИЛИ
- int **iduser** – ID пользователя (указывается ID пользователя)
- date **date** – дата счета (формат - YYYY-MM-DD)
- date **date.plan** – ожидаемая дата оплаты счета (формат - YYYY-MM-DD)
- float **summa** – сумма счета, если не указана - берем сумму Сделки
- string **invoice** - номер счета, если пусто, то будем генерировать очередной из системы
- string **contract** - номер договора, если пусто, то берем Договор, прикрепленный к сделке
- int **rs** - идентификатор расчетного счета (список можно получить из справочника) - если пусто, то берем р/с компании, от которой ведется сделка с признаком "по-умолчанию"
- int **signer** - id подписанта компании
- float **nds** - размер НДС в абсолютных цифрах (не обязательно при наличии спецификации)
- string **tip** - тип счета
    - ipre - Предварительная оплата,
    - iafter - Окончательная оплата,
    - ispeka - По спецификации,
    - icontract - По договору,
    - ioffer - Счет-договор
- string **template** - файл шаблона (см. запрос "templates") или
- int **templateID** - ID шаблона (см. запрос "templates")

**Пример запроса:**

```http request
POST http://crm_url/developer/v3/invoice/express
Content-Type: application/json
apikey: t1xdeOwWSIqgDol70CkRdK3WD4N4cm
login: vladislav@isaler.ru

{
	"did": 812,
	"tip": "ispeka",
	"invoice": "auto",
	"newstep": "60"
}
```

**Ответ:**

В поле “result“ приходит ответ обработчика, в поле “data” приходит расширенный отчет обработки

```json
{
	"result": "Успешно;Внесена оплата 70 765,00 р.<br>На р\/с Основной внесена оплата в размере 70765 р.<br>Счет добавлен в платежи",
	"data": {
		"id": 1014,
		"invoice": "261"
	},
	"messages": [
		"Внесена оплата 70 765,00 р.",
		"На р\/с Основной внесена оплата в размере 70765 р.",
		"Счет добавлен в платежи",
		"Этап сделки изменен на 60%. Предыдущий этап 40%."
	]
}
```

**Возможные ответы в случае ошибок:**

    403 – Счет по номеру не найден
    406 – Уже выставлены все возможные счета

### Запрос “mail” <a id="mail"></a>

Позволяет отправить существующий счет в виде PDF контактам по сделке

**Параметры запроса:**

- int **id** – уникальный идентификатор счета ИЛИ
- int **invoice** - номер счета
- int **did** – уникальный идентификатор сделки в CRM (обязательно в случае указания только Номера счета)
- string **user** – логин Ответственного ИЛИ
- int **iduser** – ID пользователя (указывается ID пользователя)
- string **theme** - тема сообщения, если не указано, то будет установлено "Счет на оплату"
- string **content** - тело сообщения, если не указано, то будет использован шаблон по-умолчанию

> ### Примечание
> 
> - Сначала будет произведен поиск Контактов, прикрепленным к Сделке
> - Если Контакты в сделке не найдены, то будет произведен поиск Основного контакта Клиента
> - Если Контакты не найдены, то отправка будет произведена на Email, указанный в карточке Плательщика по сделке

**Шаблон сообщения:**

<pre><code class="html">
Приветствую, {{person}}

Отправляю Вам Счет на оплату.

Спасибо за внимание.
С уважением,
{{mName}}
Тел.: {{mPhone}}
Email.: {{mMail}}
==============================
{{mCompany}}

</code></pre>

**Применимые тэги:**

- **{{person}}** - заменяет ИМЯ контакта
- **{{mName}}** - Имя сотрудника
- **{{mPhone}}** - Телефон сотрудника
- **{{mMail}}** - Email сотрудника
- **{{mCompany}}** - Название компании (короткое) из справочника "Мои компании", за которой закреплен р/сч. по счету

Указанные теги поддерживаются и для Темы сообщения

**Пример запроса:**

```http request
GET http://crm_url/developer/v3/invoice/mail
Content-Type: application/json
apikey: t1xdeOwWSIqgDol70CkRdK3WD4N4cm
login: vladislav@isaler.ru

{
	"id": 1026,
	"iduser": 20,
	"theme": "Счет на оплату оборудования от {{mCompany}}"
}
```

**Ответ:**

В поле “result“ приходит ответ обработчика, в поле “data” приходит расширенный отчет обработки

```json
{
	"result": "Успешно",
	"data": {
		"id": 1012,
		"invoice": "259"
	}
}
```

**В случае ошибки:**

```json
{
	"result":"Error",
	"error":{
        "code": 407,
        "text": "Не указан ни один получатель"
    },
	"data": 902,
    "invoice": "158",
	"text":"Выполнено с ошибками. Ошибка: Пожалуйста, введите хотя бы один адрес e-mail получателя."
}
```

**Возможные ответы в случае ошибок:**

    403 – Счет по номеру не найден
    407 – Не указан ни один получатель

### Запрос “html” <a id="html"></a>

Запрос позволяет получить содержимое счета в виде HTML

**Параметры запроса:**

- int **id** - идентификатор записи счета (если номер не уникален или не указан) ИЛИ
- string **invoice** - номер счета
- int **did** – уникальный идентификатор сделки в CRM (обязательно в случае указания только Номера счета)
- bool **nosignat** - если true, то в ответе не будет изображения печати

Наличие параметра "invoice" ИЛИ "id" обязательно для поиска счета в базе. Получить список счетов с "id" и "invoice" можно с помощью метода **info**

**Пример запроса:**

```http request
GET http://crm_url/developer/v3/invoice/html
Content-Type: application/json
apikey: t1xdeOwWSIqgDol70CkRdK3WD4N4cm
login: vladislav@isaler.ru

{
	"invoice": 277,
    "did": 812
}
```

**Ответ:**

- в поле “html“ приходит ответ HTML, 
- в поле “id” приходит id счета,
- в поле "invoice" приходит номер счета


**Возможные ответы в случае ошибок:**

    403 – Счет по ID или Номеру не найден

### Запрос “pdf” <a id="pdf"></a>

Запрос позволяет получить ссылку на счет в виде PDF

**Параметры запроса:**

- int **id** - идентификатор записи счета (если номер не уникален или не указан) ИЛИ
- string **invoice** - номер счета
- int **did** – уникальный идентификатор сделки в CRM (обязательно в случае указания только Номера счета)
- bool **nosignat** - если true, то в ответе не будет изображения печати

Наличие параметра "invoice" ИЛИ "id" обязательно для поиска счета в базе. Получить список счетов с "id" и "invoice" можно с помощью метода **info**

> ### Примечание
> 
> - если файл не существует, то он будет сгенерирован (требуется больше времени на обработку). Поэтому в cURL запросе следует установить повышенное время ожидания ответа (подбирается опытным путем, но >=20)

**Пример запроса:**

```http request
GET http://crm_url/developer/v3/invoice/pdf
Content-Type: application/json
apikey: t1xdeOwWSIqgDol70CkRdK3WD4N4cm
login: vladislav@isaler.ru

{
	"invoice": 277,
	"did": 812
}
```

**Ответ:**

- в поле “url“ приходит URL pdf-файла (слэши экранированы), 
- в поле “id” приходит id счета,
- в поле "invoice" приходит номер счета

```json
{
	"url": "http:\/\/crm_url\/files\/invoice_1030.pdf",
	"id": 1030,
	"invoice": "277"
}
```

**Возможные ответы в случае ошибок:**

    403 – Счет по ID или Номеру не найден

### Запрос “templates” <a id="templates"></a>

Запрос позволяет получить список шаблонов Счетов.

**Пример запроса:**

```http request
GET http://sm2021.crm/developer/v3/invoice/templates
Content-Type: application/json
apikey: t1xdeOwWSIqgDol70CkRdK3WD4N4cm
login: vladislav@isaler.ru
```

**Ответ:**

- **id** - идентификатор шаблона
- **title** - название шаблона
- **file** - файл шаблона
- **typeid** - идентификатор типа документа ( templateTypeID )

```json
{
	"result": "Success",
	"data": [
		{
			"id": 33,
			"title": "Базовый шаблон",
			"file": "invoice.tpl",
			"typeid": 94
		},
		{
			"id": 151,
			"title": "Квитанция на оплату",
			"file": "INV5f08d04f6257b_invoice.tpl",
			"typeid": 94
		},
		{
			"id": 152,
			"title": "С материалами",
			"file": "INV5f134900871a1_invoice.tpl",
			"typeid": 94
		},
		{
			"id": 155,
			"title": "Счет с QRcode",
			"file": "invoice_qr.tpl",
			"typeid": 94
		}
	]
}
```

**Возможные ответы в случае ошибок:**

    404 – Шаблоны не найдены
