# Заявки (lead)

## Метод “lead”

Метод позволяет передавать заявки с сайта в CRM

URL для вызова:
```http
http(s)://{{baseurl}}/developer/v3/lead/запрос?параметр=значение
```


<a id="info"></a>
### Запрос “info”

Запрос служит для вывода информации по id заявки:

Пример запроса:

```http request
GET http://{{baseurl}}/developer/v3/lead/info
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru

{
	"id": 1789
}
```

**Ответ:**

В поле “data” приходит список записей, который содержит следующие данные:

- **id** - идентификатор записи
- **datum** - дата добавления в формате ГГГГ-ММ-ДД
- **datum_do** - дата обработки в формате ГГГГ-ММ-ДД
- **данные из заявки:**
  - email
  - phone
  - site
  - company
  - city
  - country
- **content** - содержимое пришедшей заявки в виде текста
- **clid** - идентификатор записи клиента или 0, если не обработан
- **client** - название клиента
- **pid** - идентификатор записи контакта или 0, если не обработан
- **person** - фио контакта
- **did** - идентификатор записи сделки или 0, если не обработан
- **deal** - название сделки
- **user** - идентификатор записи пользователя или 0, если не обработан
- **userName** - фио пользователя
- **status** - числовой статус заявки ('0' => 'Открыт', '1' => 'В работе', '2' => 'Обработан', '3' => 'Закрыт')
- **statusName** - расшифровка статуса заявки
- **rezult** - числовой результат обработки ('1' => 'Спам', '2' => 'Дубль', '3' => 'Другое', '4' => 'Не целевой')
- **rezultName** - расшифровка результата обработки
- **clientpath** - идентификатор источника клиента (канала)
- **clientpathName** - расшифровка источника клиента

```json
{
	"data": {
		"id": 1789,
		"datum": "2021-12-06 23:45:44",
		"datum_do": "0000-00-00 00:00:00",
		"status": 0,
		"rezult": 0,
		"title": "Демо Демов",
		"email": "demo.demov@site.io",
		"phone": "+79996667788",
		"site": null,
		"company": "Демонов и Кo",
		"description": "The best crm practice",
		"ip": null,
		"city": "Perm",
		"country": "Russia",
		"iduser": 0,
		"clientpath": 86,
		"pid": 0,
		"clid": 0,
		"did": 0,
		"partner": 0,
		"utm_source": "facebook",
		"utm_medium": "cpc",
		"utm_campaign": "First Campaign",
		"utm_term": "crm для малого бизнеса",
		"utm_content": null,
		"utm_referrer": "yandex.ru",
		"uids": [
			{
				"datum": "2021-12-06 23:45:44",
				"name": "roistat_id",
				"value": "SFS864KJGBFD5",
				"eid": 0,
				"clid": 0,
				"did": 0
			}
		],
		"statusName": "Открыт",
		"rezultName": null,
		"clientpathName": "Landing Page",
		"userName": null,
		"client": null,
		"person": null,
		"deal": null
	},
	"id": 1789
}
```


<a id="list"></a>
### Запрос “list”

Запрос позволяет получить список заявок, в т.ч. с применением фильтров.

**Параметры запроса (не обязательные):**

- **offset** – страница вывода, с учетом того, что установлен лимит в 200 записей на страницу (по умолчанию offset = 0)
- **order** – поле, по которому будет производится сортировка списка (по умолчанию order = datum)
- **first** – направление сортировки (new – сначала новые, old – сначала старые). (по умолчанию first = new)

**Фильтры (не обязательные):**

- **word** - поиск по Имени, Телефону, Email и Описанию
- **status** – ограничение по статусу заявки (0 => Открыт, 1 => В работе, 2 => Обработан, 3 => Закрыт)
- **user** – ограничение по пользователю (указывается логин пользователя)
- **dateStart** – начальная дата создания записи (формат - YYYY-MM-DD)
- **dateEnd** – конечная дата создания записи (не обязательно, формат - YYYY-MM-DD)
  - только dateStart – вывод записей с датой создания больше указанной даты
  - только dateEnd – вывод записей с датой создания меньше указанной даты

**Пример запроса:**

```http request
GET http://{{baseurl}}/developer/v3/lead/list
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru

{
	"order": "datum",
	"dateStart": "2023-01-01",
	"dateEnd": "2023-01-31"
}
```

**Ответ:**

В поле “data” приходит список записей, который содержит следующие данные, аналогичные запросу **info**

<a id="stat"></a>
### Запрос “stat”

Запрос служит для вывода информации по количеству обработанных и не обработанных заявок в текущем месяце у активных пользователей системы SalesMan:

- Логин сотрудника => Открыто, Обработано

**Фильтры (не обязательные):**

- **dateStart** – начальная дата (формат - YYYY-MM-DD)
- **dateEnd** – конечная дата (не обязательно, формат - YYYY-MM-DD)
  - только dateStart – вывод записей с датой создания больше указанной даты
  - только dateEnd – вывод записей с датой создания меньше указанной даты

**Примечание:**

1. Для **Открытых** заявок учитывается дата **добавления заявки**, для **Обработанных** - дата **обработки**
2. Учитываются только активные сотрудники, указанные в качестве операторов обработки заявки

**Пример запроса:**

```http request
GET http://{{baseurl}}/developer/v3/lead/stat
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru

{
	"dateStart": "2023-01-01",
	"dateEnd": "2023-01-31"
}
```

**Ответ:**

```json
{
	"data": [
		{
			"login": "vladislav@isaler.ru",
			"open": 24,
			"processed": 94
		},
		{
			"login": "viktor@isaler.ru",
			"open": 50,
			"processed": 4
		},
		{
			"login": "demo",
			"open": 22,
			"processed": 0
		},
		{
			"login": "zahar",
			"open": 35,
			"processed": 0
		},
		{
			"login": "marand@omadaru.ru",
			"open": 78,
			"processed": 5
		},
		{
			"login": "bukin@isaler.ru",
			"open": 66,
			"processed": 7
		}
	]
}
```


<a id="add"></a>
### Запрос “add”

Запрос позволяет добавлять заявки с сайта напрямую в CRM.

**Параметры запроса:**

- **title** – Имя посетителя
- **email** – email
- **phone** – телефон
- **company** – название компании
- **description** – текстовое описание
- **ip** – ip-адрес посетителя
- **country** – страна
- **city** – город
- **path** – источник клиента
- **partner** – сайт партнера (партнер и сайт должны быть добавлены в систему отдельно)
- **user** – логин сотрудника, который назначается ответственным за обработку заявок
- **utm_source** - Источник трафика (связан со справочником "Источник клиента")
- **utm_medium** - Тип трафика
- **utm_campaign** - Рекламная кампания
- **utm_term** - Ключевая фраза
- **utm_content** - Содержание


**Примечание:**

1. Распределение заявок по сотрудникам и прочие настройки работы модуля производятся в разделе "Модули" / "Обработчик заявок" в Панели управления CRM
2. Проводится поиск по базе Клиентов и Контактов по переданным параметрам email и phone. При найденном соответствии Ответственным за обработку заявки будет назначен пользователь, ответственный за найденного Клиента и Контакт.
3. После добавления заявки Ответственному будет выслано уведомление по email

**Пример запроса:**

```http request
POST http://{{baseurl}}/developer/v3/lead/add
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru

{
	"title": "Иван Пермяков",
	"email": "ivan.permyakov@perm.io",
	"phone": "+75005007777",
	"company": "Perm Production",
	"description": "The best crm practice",
	"country": "Russia",
	"city": "Perm",
	"path": "Заявка с сайта",
	"partner": "https://yandex.ru",
	"utm_source": "facebook",
	"utm_medium": "cpc",
	"utm_campaign": "First Campaign",
	"utm_term": "crm для малого бизнеса",
	"utm_content": "",
	"utm_referrer": "yandex.ru"
}
```

**Ответ:**

В поле “data” приходит id записи

```json
{
  "result": "Success",
  "id": 1649
}
```
