# Статистика (statistic)


## Метод “statistic”

Метод позволяет получать статистику по показателям за определенный период времени с детализацией по сотрудникам:

* просмотр общей статистики
* просмотр статистики по новым клиентам с детализацией по сотрудникам
* просмотр статистики по новым и закрытым сделкам с детализацией по сотрудникам
* просмотр статистики по выставленным и оплаченным счетам с детализацией по сотрудникам

URL для вызова:
```http
http(s)://{{baseurl}}/developer/v3/statistic/запрос?параметр=значение
```

### Варианты парметра "period"

- **today** - сегодня (текущая дата), 
- **yestoday** - вчера,
- **week** - период за последние 7 дней,
- **calendarweek** - текущая календарная неделя
- **calendarweekprev** - предыдущая календарная неделя
- **month** - календарный месяц,
- **prevmonth** - предыдущий календарный месяц,
- **quart** - календарный квартал,
- **prevquart** - предыдущий календарный квартал,
- **year** - календарный год
- **yearprev** - предыдущий календарный год


> ### Примечание
> 
> Сгенерированное изображение диаграммы доступно по ссылке из ответа и актуально в течение 1 часа
> 

<a id="list"></a>
### Запрос “list”

Запрос позволяет получить статистические данные по основным показателям за выбранный период времени.

**Параметры запроса:**

- **period** – период данных
- **user** – login пользователя в SalesMan CRM
- **personal** - параметр, позволяющий выводить информацию только по руководителю, даже если у него есть подчиненные (yes | no - по умолчанию)

**Пример запроса:**

```http request
GET http://{{baseurl}}/developer/v3/statistic/list
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru

{
    "period": "yearprev"
}
```

**Ответ:**

```json
{
	"title": "Статистика",
	"period": "с 01.01.2023 по 31.12.2023",
	"periodStart": "2023-01-01",
	"periodEnd": "2023-12-31",
	"details": {
		"clients": 7,
		"deals": {
			"new": {
				"count": 5,
				"sum": 2007802.75
			},
			"close": {
				"count": 2,
				"sum": 375340
			}
		},
		"invoices": {
			"count": 10,
			"sum": 2524610.75
		},
		"payments": {
			"count": 4,
			"sum": 1339800
		}
	}
}
```

**Возможные ответы в случае ошибок:**

    403 - Неверный период времени 
    405 - Отсутствует параметр - период для отчетности


<a id="clients"></a>
### Запрос “clients”

Запрос позволяет получить информацию о количестве новых клиентов за заданный период времени с детализацией по сотрудникам.

**Параметры запроса:**

- **period** – период отчетности(обязательное поле)
- **user** - логин сотрудника
- **diagram** - отметка для построения диаграммы ("yes")
- **personal** - параметр, позволяющий выводить информацию только по руководителю, даже если у него есть подчиненные (yes | no - по умолчанию)

**Пример запроса**:

```http request
GET http://{{baseurl}}/developer/v3/statistic/clients
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru

{
	"period": "yearprev"
}
```

**Ответ**:

```json
{
	"title": "Новые клиенты",
	"period": "с 01.01.2023 по 31.12.2023",
	"periodStart": "2023-01-01",
	"periodEnd": "2023-12-31",
	"count": 7,
	"details": [
		{
			"user": "Андреев Владислав",
			"count": 7,
			"part": "100%"
		}
	]
}
```

**Возможные ответы в случае ошибок:**

    403 – Неверный период времени
    405 – Отсутствуют параметры - период для отчета


<a id="dealsNew"></a>
### Запрос “dealsNew”

Запрос позволяет получить информацию о количестве новых сделок за заданный период времени с детализацией по сотрудникам.

> ### Примечание
> 
> Данные выводятся по Автору сделки, а не по Ответственному
> 

**Параметры запроса:**

- **period**  – период отчетности(обязательное поле)
- **user**    - логин сотрудника
- **diagram** - отметка для построения диаграммы("yes")
- **personal** - параметр, позволяющий выводить информацию только по руководителю, даже если у него есть подчиненные (yes | no - по умолчанию)

**Пример запроса**:

```http request
GET http://{{baseurl}}/developer/v3/statistic/dealsNew
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru

{
	"period": "yearprev"
}
```


**Ответ**:

```json
{
	"title": "Новые сделки",
	"period": "с 01.01.2023 по 31.12.2023",
	"periodStart": "2023-01-01",
	"periodEnd": "2023-12-31",
	"count": 5,
	"details": [
		{
			"user": "Андреев Владислав",
			"count": 4,
			"summa": 1875428.5
		},
		{
			"user": "Администратор",
			"count": 1,
			"summa": 132374.25
		}
	]
}
```

**Возможные ответы в случае ошибок:**

    403 – Неверный период времени
    405 – Отсутствуют параметры - период для отчета


<a id="dealsClose"></a>
### Запрос “dealsClose”

Запрос позволяет получить информацию о количестве закрытых сделок за заданный период времени с детализацией по сотрудникам.


**Параметры запроса:**

- **period**  – период отчетности(обязательное поле)
- **user**    - логин сотрудника
- **diagram** - отметка для построения диаграммы("yes")
- **personal** - параметр, позволяющий выводить информацию только по руководителю, даже если у него есть подчиненные (yes | no - по умолчанию)

**Пример запроса**:

```http request
GET http://{{baseurl}}/developer/v3/statistic/dealsClose
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru

{
	"period": "yearprev"
}
```

**Ответ**:

```json
{
	"title": "Закрытые сделки",
	"period": "с 01.01.2023 по 31.12.2023",
	"periodStart": "2023-01-01",
	"periodEnd": "2023-12-31",
	"count": 2,
	"details": [
		{
			"user": "Андреев Владислав",
			"count": 2,
			"summa": 375340
		}
	]
}
```


**Возможные ответы в случае ошибок:**

    403 – Неверный период времени
    405 – Отсутствуют параметры - период для отчета


<a id="invoices"></a>
### Запрос “invoices”

Запрос позволяет получить информацию о количестве новых выставленных счетов за заданный период времени с детализацией по сотрудникам.

**Параметры запроса:**

- **period**  – период отчетности(обязательное поле)
- **user**    - логин сотрудника
- **diagram** - отметка для построения диаграммы("yes")
- **personal** - параметр, позволяющий выводить информацию только по руководителю, даже если у него есть подчиненные (yes | no - по умолчанию)

**Пример запроса**:

```http request
GET http://{{baseurl}}/developer/v3/statistic/invoices
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru

{
	"period": "yearprev"
}
```

**Ответ**:

```json
{
	"title": "Новые счета",
	"period": "с 01.01.2023 по 31.12.2023",
	"periodStart": "2023-01-01",
	"periodEnd": "2023-12-31",
	"count": 10,
	"details": [
		{
			"user": "Андреев Владислав",
			"count": 8,
			"summa": 2463623.5
		},
		{
			"user": "Марусин Андрей",
			"count": 2,
			"summa": 375374.25
		}
	]
}
```

**Возможные ответы в случае ошибок:**

    403 – Неверный период времени
    405 – Отсутствуют параметры - период для отчета


<a id="payments"></a>
### Запрос “payments”

Запрос позволяет получить информацию о количестве оплаченных счетов за заданный период времени с детализацией по сотрудникам.

**Параметры запроса:**

- **period**  – период отчетности(обязательное поле)
- **user**    - логин сотрудника
- **personal** - параметр, позволяющий выводить информацию только по руководителю, даже если у него есть подчиненные (yes | no - по умолчанию)
- **diagram** - отметка для построения диаграммы("yes")

**Пример запроса**:

```http request
GET http://{{baseurl}}/developer/v3/statistic/payments
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru

{
	"period": "yearprev"
}
```

**Ответ**:

```json
{
	"title": "Оплаченные счета",
	"period": "с 01.01.2023 по 31.12.2023",
	"periodStart": "2023-01-01",
	"periodEnd": "2023-12-31",
	"count": 4,
	"details": [
		{
			"user": "Андреев Владислав",
			"count": 3,
			"summa": 1034292
		},
		{
			"user": "Букин Геннадий",
			"count": 1,
			"summa": 305508
		}
	]
}
```

**Возможные ответы в случае ошибок:**

    403 – Неверный период времени
    405 – Отсутствуют параметры - период для отчета