# Напоминания (task)

## Метод “task”

Метод позволяет управлять записями Напоминаний – добавлять, обновлять, удалять.

URL для вызова:
```http
http(s)://{{baseurl}}/developer/v3/task/запрос?параметр=значение
```


Специфичные признаки, используемые далее:

- **tid** – уникальный идентификатор напоминания (в запросах идет как **id**)
- **active** – признак активности напоминания (yes - активно или no - выполнено)
- **priority** – важность (1 – не важно, 0 – нормально*, 2 – важно)
- **speed** – срочность (1 – не срочно, 0 – нормально*, 2 – срочно)
- **status** – статус выполнения (1-успешно, 2-не успешно)


<a id="fields"></a>
### Запрос “fields”

Запрос позволяет получить список доступных полей, хранящих информацию о клиенте в формате – «Имя поля» - «Расшифровка назначения» для формирования дальнейших запросов.

В список включены поля, активированные в Панели управления / Формы. Не активные поля игнорируются при обработке.

**Пример запроса:**

```http request
GET http://{{baseurl}}/developer/v3/task/fields
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru
```

**Ответ:**

```json
{
	"data": {
		"tid": "Идентификатор напоминания",
		"datum": "Дата",
		"totime": "Время",
		"title": "Тема",
		"des": "Агенда",
		"tip": "Тип напоминания",
		"active": "Признак: выполнено,активно",
		"priority": "Важность",
		"speed": "Срочность",
		"iduser": "Ответственный",
		"autor": "Автор напоминания",
		"clid": "ID клиента",
		"pid": "ID контакта",
		"did": "ID сделки",
		"readonly": "Только для чтения",
		"day": "На весь день",
		"status": "Статус выполнения"
	}
}
```


<a id="tips"></a>
### Запрос “tips”

Запрос позволяет получить список доступных типов дел и их цвета для формирования дальнейших запросов.

**Пример запроса:**

```http request
GET http://{{baseurl}}/developer/v3/task/tips
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru
```

**Ответ:**

Параметр **filter** в ответе означает принадлежность активности:

1. **all** - относится и к Напоминаниям и к Истории активностей
2. **activ** - относится только к Истории активностей
3. **task** - относится только к Напоминаниям

```json
{
	"data": [
		{
			"title": "Первичный звонок",
			"color": "#92cddc",
			"filter": "all"
		},
		{
			"title": "Факс",
			"color": "#cc00cc",
			"filter": "activ"
		},
		{
			"title": "Встреча",
			"color": "#31859b",
			"filter": "all"
		},
		{
			"title": "Задача",
			"color": "#ff6600",
			"filter": "all"
		},
		{
			"title": "Предложение",
			"color": "#66ccff",
			"filter": "activ"
		},
		{
			"title": "Событие",
			"color": "#666699",
			"filter": "activ"
		},
		{
			"title": "исх.Почта",
			"color": "#cccc00",
			"filter": "all"
		},
		{
			"title": "вх.Звонок",
			"color": "#99cc00",
			"filter": "all"
		},
		{
			"title": "вх.Почта",
			"color": "#cc3300",
			"filter": "all"
		},
		{
			"title": "Поздравление",
			"color": "#009999",
			"filter": "task"
		},
		{
			"title": "исх.2.Звонок",
			"color": "#339966",
			"filter": "all"
		},
		{
			"title": "Отправка КП",
			"color": "#ff0000",
			"filter": "all"
		},
		{
			"title": "Исходящий звонок КЦ",
			"color": "#3498DB",
			"filter": "activ"
		},
		{
			"title": "Холодный звонок",
			"color": "#99ccff",
			"filter": "activ"
		},
		{
			"title": "Вакансия",
			"color": "#c67c00",
			"filter": "all"
		},
		{
			"title": "",
			"color": "",
			"filter": "all"
		}
	]
}
```


<a id="info"></a>
### Запрос “info”

Запрос позволяет получить информацию по напоминанию по его идентификатору - tid.

**Параметры запроса:**

- **id** – уникальный идентификатор записи напоминания

**Пример запроса:**

```http request
GET http://{{baseurl}}/developer/v3/task/info
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru

{
    "id": 3492
}
```

**Ответ:**

```json
{
	"data": {
		"tid": "3492",
		"datum": "2023-06-29",
		"totime": "14:00",
		"title": "Договориться о встрече",
		"des": "Использование данной настройки позволяет вести учет сделок, оплат и расходов с распределением по различным собственным компаниям. Поддерживается неограниченное количество собственных (дочерних) компаний, а также любое количество расчетных счетов (банковских или касс наличных средств).",
		"tip": "Задача",
		"active": "yes",
		"priority": "0",
		"speed": "0",
		"iduser": 1,
		"user": "vladislav@isaler.ru",
		"autor": 0,
		"autorlogin": "",
		"clid": 1781,
		"client": "Сейлзмен Рус",
		"person": [
			{
				"pid": 2513,
				"person": "Васильев Дмитрий"
			},
			{
				"pid": 2475,
				"person": "Андреев Владислав"
			}
		],
		"did": 0,
		"dogovor": "",
		"readonly": "no",
		"day": null,
		"status": "0"
	}
}
```

**Примечание:**

1. Если в ответе autor = 0, то автором является сам ответственный
2. Контакты возвращаются в виде массива

**Возможные ответы в случае ошибок:**

    403 – Напоминание с указанным id не найдено в пределах аккаунта указанного пользователя
    404 – Не найдено
    405 – Отсутствуют параметры - id напоминания


<a id="list"></a>
### Запрос “list”

Запрос позволяет получить список напоминаний, доступных текущему сотруднику, в т.ч. с применением фильтров.

**Параметры запроса (не обязательные):**

- **offset** – страница вывода, с учетом того, что установлен лимит в 200 записей на страницу (по умолчанию offset = 0)
- **order** – поле, по которому будет производится сортировка списка (по умолчанию order = datum)
- **first** – направление сортировки (new – сначала новые, old – сначала старые ). (по умолчанию first = new)

**Фильтры (не обязательные):**

- **user** – ограничение по пользователю (указывается логин пользователя)
- **dateStart** – начальная дата напоминания (формат - YYYY-MM-DD)
- **dateEnd** – конечная дата напоминания (не обязательно, формат - YYYY-MM-DD)
  - только dateStart – вывод записей с датой выполнения больше указанной даты
  - только dateEnd – вывод записей с датой выполнения меньше указанной даты
- **word** – слово поиска по полям title, des, tip
- **tip** – поиск по определенному типу напоминания (title в ответе запроса tips)
- **active** - фильтр по статусу выполнения: yes - активное, no - выполненное
- **clid** - фильтр по Клиенту
- **did** - фильтр по Сделке
- **status** - фильтр по статусу выполнения (1-успешно, 2-не успешно)

**Примечание:**

1. По умолчанию выводятся напоминания на текущий день
2. Если не указан **user**, то будут выведены записи текущего пользователя и всех подчиненных
2. Если указан статус выполнения **status**, то будут выведены записи только выполненных напоминаний

**Пример запроса:**

```http request
GET http://{{baseurl}}/developer/v3/task/list
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru

{
	"dateStart": "2023-01-01",
	"dateEnd": "2023-06-31",
	"active": "yes",
    "did": 975
}
```

**Ответ:**

- Ответ аналогичен запросу **info**

**Возможные ответы в случае ошибок:**

    400 – Не верный API key
    401 – Неизвестный пользователь
    402 – Неизвестный метод


<a id="add"></a>
### Запрос “add”

Запрос позволяет добавить новое напоминание в базу CRM. При этом ответственным устанавливается сотрудник, логин которого использовался в запросе или указанный отдельно в параметре user

**Параметры запроса:**

- **user** – login пользователя в SalesMan CRM, которому назначается напоминание
- **datum** => Дата в формате YYYY-MM-DD (по умолчанию текущая дата)
- **totime** => Время в формате H:i (по умолчанию текущее время + 1 час)
- **title** => Тема
- **des** => Агенда
- **tip** => Тип напоминания
- **priority** => Важность
- **speed** => Срочность
- **autor** => логин Автора напоминания (по умолчанию текущий пользователь)
- **clid** => ID клиента
- **pid** => ID контакта
- **did** => ID сделки

**Примечание:**

1. при пустом поле **user** Ответственным будет назначен текущий пользователь (из запроса)
2. уведомление будет отправлено пользователю в зависимости от его настроек
3. следующие параметры передаются в явном, текстовом виде:
   - **tip** – Тип напоминания
   - В случае отсутствия переданных значений в справочниках будут созданы новые записи типа Активности

**Пример запроса:**

```http request
POST http://{{baseurl}}/developer/v3/task/add
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru

{
	"datum": "2024-07-01",
	"totime": "15:30",
	"title": "Перезвонить клиенту с сайта",
	"des": "Это описание",
	"tip": "Задача",
	"priority": 2,
	"speed": 0,
	"clid": 1781,
	"pid": "2475,2723",
	"did": 784
}
```


**Ответ:**

В поле “data” приходит id созданной записи

```json
{
	"result": "Успешно",
	"data": 3497
}
```

**Возможные ответы в случае ошибок:**

    405 – Отсутствуют параметры


<a id="update"></a>
### Запрос “update”

Запрос позволяет обновить данные Напоминания по его tid. При этом нет необходимости передавать все данные – можно передать только изменившиеся данные.

**Параметры запроса:**

- **tid** – уникальный идентификатор контакта (обязательное поле)
- прочие поля fields – информация для обновления
    
**Пример формирования запроса в PHP:**

```http request
POST http://{{baseurl}}/developer/v3/task/update
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru

{
    "id": 3497,
	"datum": "2024-07-02",
	"totime": "15:30",
	"priority": 2,
	"speed": 2
}
```

**Ответ:**

```json
{
	"result": "Успешно",
	"data": 3497
}
```

**Возможные ответы в случае ошибок:**

    403 – Напоминание не найдено
    405 – Отсутствуют параметры - id напоминания


<a id="doit"></a>
### Запрос “doit”

Запрос позволяет отметить напоминание выполненным по его id. При этом можно зафиксировать результат выполнения.

**Параметры запроса:**

- **id** – уникальный идентификатор контакта (обязательное поле)
- **description** – результат выполнения напоминания
- **tip** - тип события для внесения в Историю активностей. Если не указано, будет взято из напоминания
- **status** – статус выполнения (1-успешно,2-не успешно)

**Пример запроса:**

```http request
POST http://{{baseurl}}/developer/v3/task/doit
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru

{
	"id": 3497,
	"description": "Я перезвонил клиенту. Круто-чо!",
	"tip": "Исх.звонок",
    "status": 1
}
```

**Ответ:**

```json
{
	"result": "Успешно",
	"data": 3497,
	"message": "Добавлена Активность",
	"historyID": 25956
}
```

**Возможные ответы в случае ошибок:**

    403 – Напоминание не найдено
    405 – Отсутствуют параметры - id напоминания


<a id="delete"></a>
### Запрос “delete”

Запрос позволяет удалить напоминание по его id.

**Параметры запроса:**

- id – уникальный идентификатор напоминания (обязательное поле)

**Пример запроса:**

```http request
DELETE http://{{baseurl}}/developer/v3/task/delete
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru

{
	"id": 3498
}
```

**Ответ:**

```json
{
	"result": "Успешно",
	"data": 3498,
	"message": "Запись удалена"
}
```

**Возможные ответы в случае ошибок:**

    403 – Напоминание не найдено
    405 – Отсутствуют параметры - id напоминания