# Сотрудники (user)

## Метод “user”

Метод позволяет работать с записями сотрудников.

URL для вызова:
```http
http(s)://{{baseurl}}/developer/v3/user/запрос?параметр=значение
```


<a id="user"></a>
### Запрос “user”

Запрос позволяет получить расширенню информацию о Пользователе аккаунта

**Параметры запроса:**

- **user** – Логин пользователя
- **uid** – уникальный идентификатор, связанный с внешней ИС или
- **iduser** - уникальный идентификатор в CRM

Можно указать либо uid (он должен быть добавлен в систему), либо iduser требуемого пользователя. Первоначально проверяется uid.

```http request
GET http://{{baseurl}}/developer/v3/user/info
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru

{
    "iduser": 1
}
```

**Ответ:**

Полученный ответ содержит данные:

- **iduser** - уникальный идентификатор в CRM
- **uid** – уникальный идентификатор, связанный с внешней ИС
- **oid** – уникальный идентификатор отдела, связанный с внешней ИС
- **title** - имя Пользователя
- **login** – логин Пользователя
- **active** – Статус пользователя (yes – активен, no – заблокирован)
- **tip** - роль в системе
- **user_post** - должность
- **email** - email пользователя (для пользователей нашего сервиса совпадает с логином)
- **phone** - телефон
- **phone_in** - внутренний (добавочный) номер (для sip-аккаунтов)
- **mob** - мобильный телефон

```json
{
	"data": {
		"iduser": 1,
		"uid": "DIR100",
		"oid": "OAP",
		"title": "Андреев Владислав",
		"login": "vladislav@isaler.ru",
		"active": "yes",
		"tip": "Руководитель подразделения",
		"user_post": "Руководитель проекта",
		"email": "vladislav@isaler.ru",
		"phone": "7(495)373-07-63",
		"phone_in": "771",
		"mob": "+79223289467"
	}
}
```

<a id="user.list"></a>
### Запрос “list”

Запрос позволяет получить список Пользователей аккаунта

```http request
GET http://{{baseurl}}/developer/v3/user/list
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru
```

**Ответ:**

Полученный ответ содержит данные по каждому сотруднику:

- **iduser** - уникальный идентификатор в CRM
- **uid** – уникальный идентификатор, связанный с внешней ИС
- **oid** – уникальный идентификатор отдела, связанный с внешней ИС
- **title** – Имя пользователя
- **login** – Логин пользователя
- **email** – Email пользователя
- **active** – Статус пользователя (yes – активен, no – заблокирован)


<a id="add"></a>
### Запрос “add”

Запрос позволяет добавить нового Пользователя

**Параметры запроса:**

- **uid** – уникальный идентификатор, связанный с внешней ИС
- **user** – Логин пользователя (для SalesMan Online это email)
- **password** - Пароль пользователя
- **title** – Имя пользователя
- **mid** - Идентификатор (uid) руководителя Пользователя в вашей системе (должен совпадать с идентификатором в CRM)
- **otdel** - Идентификатор (oid) отдела во внешней системе
- **tip** - роль в системе (Руководитель организации, Руководитель с доступом, Руководитель подразделения, Руководитель отдела, Менеджер продаж, Поддержка продаж, Специалист, Администратор)
- **user_post** - должность
- **email** - email пользователя (для пользователей нашего сервиса совпадает с логином)
- **phone** - телефон
- **phone_in** - внутренний телефон (для sip-аккаунтов)
- **mob** - мобильный телефон
- **boss** - логин руководителя

```http request
POST http://{{baseurl}}/developer/v3/user/add
Content-Type: application/json
apikey: {{token}}
login: vladislav@isaler.ru

{
	"user": "ivana.petrova@isaler.ru",
	"password": "PassW0Rd!2",
	"title": "Ивана Петрова",
	"tip": "Поддержка продаж",
	"user_post": "Ассистент менеджера",
	"bday": "2004-02-02",
	"phone": "",
	"phone_in": "",
	"mob": "",
	"email": "ivana.petrova@isaler.ru",
	"boss": "vladislav@isaler.ru",
	"otdel": "OAP"
}
```

**Ответ:**

В случае успешного запроса вернется iduser в поле data

```json
{
	"result": "Success",
	"data": 79
}
```

**Возможные ответы в случае ошибок:**

    406 - Такой логин уже существует
    500 - Внутренняя ошибка
