# О плагине

 * @package     **{{package}}**
 * @author      **Vladislav Andreev** <v@salesman.pro>
 * @version     v.**{{version}}** от {{versiondate}}

Плагин позволяет управлять заданиями для планировщика CRON из удобного веб-интерфейса.

## Changelog

28.02.2023: **v1.1**
- накопительные исправления

05.01.2023: **v1.0**
- накопительные исправления
- добавлен скрипт "Тотальная очистка"
- добавлен скрипт "Разморозка сделок"

03.08.2022: **v0.6 beta**
- накопительные исправления
- добавлена возможность передавать параметры
- добавлен метод disableTask

03.02.2020: **v0.5 beta** 
- первый релиз beta-версии


# Возможности плагина

* Управление заданиями CRONTAB в удобном интерфейсе
* Человеко-понятное формирование периодичности выполнения задания
* Просмотр логов выполнения заданий ( последние 10 результатов для каждого задания )
* Отключение задания без необходимости удаления
* Контроль выполнения задания ( для длительных заданий ) - новое задание не будет запущено, если не закончено выполнение предыдущего запуска этого же задания


# Настройка плагина

## Запуск основного процесса

Для успешного функционирования плагина следует добавить единственное задание в стандартный планировщик заданий:

{{maincrontask}}

Именно этот скрипт будет управлять всеми остальными заданиями.

> #### Важно!
> Замените path/to/phpbin на свой путь до исполняемого файла PHP
> {{phppathText}}


## Формирование заданий

Формирование заданий состоит из 3-х составляющих:

* периодичность
* исполняемая программа - wget, curl, php и т.п.
* исполняемая команда (скрипт) - наш php-скрипт, который будем выполнять

Подробнее про Cron - [https://ru.wikipedia.org/wiki/Cron](https://ru.wikipedia.org/wiki/Cron)

### Исполняемая программа

Исполняемая программа ( ИП ) - это основная программа, которая выполняет указанную задачу, записанную в виде скрипта. В качестве ИК могут использоваться:

- wget
- curl
- php
- bash
- и т.п.

Наши скрипты написаны на PHP, поэтому будем рассматривать его в качестве исполняемой программы:
- Минимальная версия PHP, поддерживаемая плагином 7.2
- Если на сервере установлена одна версия PHP, то можно указать просто **php**
- Если на сервере установлены альтернативные версии PHP, то рекомендуется выяснить путь до исполняемого файла командой **locate -b '\php'** ( например **_/opt/php72/bin/php_** )

> #### Важно!
> Утилиты WGET и CURL используют HTTP-запросы (как браузер), поэтому ИП должна содержать URL-адрес.
> 
> Например: 
> 
> ```
> * * * * * /usr/bin/wget -O - -q -t 1 --no-check-certificate https://crm.ru/api/leads/cron.php?api_key=t1xdeOwWSIqgDol70CkRdK3WD4N4cm /dev/null 2>&1
> ```

### Исполняемая команда ( ИК )

Исполняемая команда - это наш скрипт, который выполняет необходимые действия. Плагин может подключить существующие системные скрипты, разработанные для планировщика заданий с рекомендованными настройками по периодичности:

* **Бекап Базы данных** - производит ежедневное резервное копирование БД
* **Дела на день** - производит отправку дел на текущий день, рекомендуется запуск в утреннее время
* **Проверка почты сотрудников** - производит проверку почты через Почтовик для каждого сотрудника, не зависимо от того - авторизовался пользователь в системе или нет
* **Сборщик заявок** - производит проверку почтовых ящиков, настроенных в модуле Сборщик заявок
* **Синхронизация истории звонков** (для всех поддерживаемых PBX) - производит синхронизацию истории звонков
* **Тотальная очистка** - скрипт производит глубокую очистку от ненужных записей
	- старые письма и не нужные вложения
	- историю звонков - внутренние старше 30 дней и не привязанные звонки старше 6 месяцев
	- старые записи различных логов (веб-хук, api, ...)
	- напоминания старше 1 года
	- активности старше 3х лет или с пустым содержимым
	- активности с типом ЛогCRM, СобытиеCRM старше 180 дней
* **Разморозка сделок** - размораживает сделки и ставит напоминания по ним для ответственных

### Периодичность

Плагин поддерживает 6 периодов запуска + 1 экспертный режим ( для тех, кто знаком с командами ):

1. **Каждую минуту** - в этом режиме задается периодичность в минутах выполнения задания. Например: каждые Х минут
2. **Каждый час** - в этом режиме задание выполняется каждые Х часов
3. **Ежедневно** - в этом режиме задание выполняется в Х часов Y минут ежедневно
4. **Еженедельно** - в этом режиме задание выполняется в указанное время Х часов Y минут в указанные дни недели
5. **Ежемесячно** - в этом режиме задание выполняется в указанное время Х часов Y минут в указанные дни каждого месяца
6. **Ежегодно** - в этом режиме задание выполняется в указанное время Х часов Y минут в указанный день указанного месяца
7. **Экспертный режим** - в этом режиме есть возможность задать задание в явном виде

## Использование в сторонних модулях и плагинах

Для добавления своего задания в Планировщик надо подключить необходимые классы:

```php
require_once $rootpath."/plugins/cronManager/php/autoload.php";
require_once $rootpath."/plugins/cronManager/vendor/autoload.php";

$cron = new Cronman\Cronman();

$task = [
    // название задания
    "name" => "customName",
    // стандартная периодичность: см. $cron::PERIODS;
    "parent" => "everyminutes",
    // исполняемая программа
    "bin" => "/opt/php72/bin/php",
    // исполняемый скрипт, где {{DIR}} будет заменен на 
	// абсолютный путь до папки с CRM,
    // или указываем свой путь или https-адрес
    "script" => "{{DIR}}/cron/backup.php",
    // массив с указанием периодичности
    "period" => [
        "i" => "",
        "h" => "",
        "d" => "",
        "m" => "",
        // для экспертного режима, например 0 */1 * * *
        "cmd" => "",
    ],
    // передача любых параметров в виде json
    "task" => "{\"UserID\":\"1\",\"da1\":\"2022-08-01\",\"da2\":\"2022-08-07\"}",
    // активность задания - on|off
    "active" => "on",
];

// id задания приходит в аргументах
$taskID   = (int)$argv[1];

// добавляет/редактирует задание и 
// возвращает id записи задания в базе данных
$shedule = $cron -> setTask($taskID, $task);

// возвращает данные по заданию
$shedule = $cron -> getTask($taskID);

// удаляет задание
$result = $cron -> deleteTask($taskID);

// деактивация задания
$result = $cron -> disableTask($taskID);

// возвращает список заданий
$result = $cron -> getTaskList();
```

## Сценарий применения для больших задач

Некоторые задачи требуют длительного времени выполнения скриптов. При этом возникает ошибка 504 Gateway Time-out.

Что можно сделать? Выполнять скрипты php напрямую, т.е. без участия веб-сервера. И в этом поможет плагин cronManager.

Как это работает:

1. Создаем задание. Например:
* parent = once - признак однократного исполнения задания
```php
$task = new Cronman();
$task -> setTask(0, [
    "uid"    => time(),
    "name"   => "My First Task",
    "bin"    => "php",
    "script" => "/path_to_script/script.php",
    "task"   => json_encode_cyr($params),
    "active" => "on",
    "parent" => "once"
]);
```
2. В вызываемом скрипте принимаем параметры:
- параметры приходят в аргументе
```php
$taskID   = (int)$argv[1];
```
- извлекаем параметры
```php
$cron   = new Cronman();
$task   = $cron -> getTask($taskID);
$params = json_decode($task['task'], true);
```
- деактивируем задание, чтобы планировщик не запускал его повторно каждую минуту
```php
$cron -> disableTask( $taskID );
```
- выполняем свою задачу
```php
// тут ваш код
```
- удаляем задание
```php
$cron -> deleteTask($taskID);
```